function X_new = DecodeKalmanSS(X_old, Y_new, filtParams)
% Steady-state Kalman filter using eigenvector method. This function
% executes a single recursion of the filter to decode a single vector of
% noisy observations. To decode a time series of observations, this 
% function should be contained inside a loop.
% Typically, one would use a set of training data to calibrate the filter
% first, then use a set of test data to decode a series of observations
% using the calibrated filter and estimate the hidden states. 
% See [1] for details.
%
% Inputs:
%   X_old       State vector from previous Kalman recursion (time t-1) 
%               used to compute the state estimate in the time-update step 
%               (Sx1 vector, where S = no. of states at each time-step)
%   Y_new       Current (time t) bservation vector (Mx1 vector, where
%               M = dimensionality of observations at each time-step)
%   filtParams  The parameters of the filter calibrated by the function
%               CalibrateKalmanSS, namely matrices A, C, Q, R, and also
%               the steady-state K and P 
% Outputs:
%   X_new       Decoded state vector (Sx1 vector, where S = no. of states)
%
% Wasim Q. Malik - June 29, 2009 <wqm@mit.edu>
% 
%*************************************************************************
% IMPORTANT NOTICE: If you use this code in your work, please cite [1].
%*************************************************************************


%*************************************************************************
%   Set local variables
    K_ss = filtParams.K;
    A = filtParams.A;
    C = filtParams.C;

%*************************************************************************
%   Decoding: Apply Kamlan filter model for prediction
        Xhatbar = A * X_old;                             % a priori estimate of posterior state 
        X_new = Xhatbar + K_ss * (Y_new - C * Xhatbar);  % posterior state estimate

%*************************************************************************
%   References
%   [1] W. Q. Malik, W. Truccolo, E. N. Brown, and L. R. Hochberg, 
%       "Efficient decoding with steady-state Kalman filter in neural
%       interface systems," IEEE Transactions on Neural Systems and
%       Rehabilitation Engineering, vol. 19, no. 1, pp.25-34, Feb. 2011
